package main

import (
	"context"
	"encoding/json"
	models "eta/eta_crawler/models"
	"eta/eta_crawler/utils"
	"fmt"
	"github.com/beego/beego/v2/core/logs"
	"github.com/chromedp/cdproto/cdp"
	"os"
	"strings"
	"time"

	"github.com/chromedp/chromedp"
)

const (
	lyLoginPath = "https://www.fao.com.cn/"
)

// UserInfo 登录用户信息
type UserInfo struct {
	Username string
	Password string
}

func main() {

	// 读取 JSON 文件
	configFile, err := os.ReadFile("D:\\go\\workspace1\\eta_crawler\\static\\liangyou.json")
	if err != nil {
		fmt.Printf("读取配置文件错误: %v\n", err)
		return
	}

	// 定义通用的 map 结构体来解析 JSON
	var data map[string]map[string]map[string][]string

	// 解析 JSON 文件内容
	err = json.Unmarshal(configFile, &data)
	if err != nil {
		fmt.Printf("解析配置文件错误: %v\n", err)
		return
	}

	// 打印解析后的数据以验证
	fmt.Printf("%+v\n", data)

	// 创建 chromedp 执行上下文
	options := []chromedp.ExecAllocatorOption{
		chromedp.Flag("headless", false),
		chromedp.Flag("disable-blink-features", "AutomationControlled"),
		chromedp.UserAgent(`Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/98.0.4758.80 Safari/537.36`),
	}

	allocCtx, cancel := chromedp.NewExecAllocator(context.Background(), options...)
	defer cancel()
	ctx, cancel := chromedp.NewContext(allocCtx)
	defer cancel()

	// 登录操作
	err = login(ctx)
	if err != nil {
		fmt.Printf("登录错误: %v\n", err)
		return
	}

	// 遍历配置并爬取数据
	for product, productData := range data {
		for category, categoryData := range productData {
			for report, keywords := range categoryData {
				fmt.Printf("正在获取数据: %s -> %s -> %s\n", product, category, report)
				err = fetchReportData(ctx, product, category, report, keywords)
				if err != nil {
					fmt.Printf("获取数据错误: %s -> %s -> %s: %v\n", product, category, report, err)
				}
			}
		}
	}
}

func login(ctx context.Context) error {
	userInfo := UserInfo{
		Username: "13633849418",
		Password: "828384Abc@",
	}
	return chromedp.Run(ctx,
		chromedp.Navigate(lyLoginPath),
		chromedp.Sleep(5*time.Second),
		chromedp.Click(`a[id="btnLogin"]`, chromedp.ByQuery),
		chromedp.Sleep(2*time.Second),
		chromedp.SetValue(`input[id="userName"]`, userInfo.Username, chromedp.ByQuery),
		chromedp.SetValue(`input[id="pwd"]`, userInfo.Password, chromedp.ByQuery),
		chromedp.Sleep(2*time.Second),
		chromedp.Click(`input[id="btn_Login"]`, chromedp.ByQuery),
		chromedp.Sleep(5*time.Second),
	)
}

func fetchReportData(ctx context.Context, product, category, report string, keywords []string) error {
	// Navigate to the main page
	err := chromedp.Run(ctx,
		chromedp.Navigate(lyLoginPath),
		chromedp.Sleep(5*time.Second),
	)
	if err != nil {
		return err
	}

	// Navigate to the product page
	productPageURL, err := fillProductPageURL(ctx, product, category)
	if err != nil {
		return err
	}

	// Navigate to the category page
	var categoryPageURL string
	err = chromedp.Run(ctx,
		chromedp.Navigate(productPageURL),
		chromedp.Sleep(5*time.Second),
		chromedp.Click(fmt.Sprintf(`//div[contains(@class, "newBox")]//a[contains(text(), '%s')]`, category), chromedp.BySearch),
		chromedp.Sleep(5*time.Second),
		chromedp.Location(&categoryPageURL),
	)
	if err != nil {
		return err
	}
	logs.Info("categoryPageURL: %s: %s: %s", product, category, categoryPageURL)

	var allReportURLs []string
	for {
		var htmlContent string
		err = chromedp.Run(ctx,
			chromedp.Navigate(categoryPageURL),
			chromedp.Sleep(5*time.Second),
			chromedp.OuterHTML("html", &htmlContent),
		)
		if err != nil {
			return err
		}
		fmt.Printf("页面内容: %s\n", htmlContent)

		// Extract report URLs containing the partial keyword
		reportURLs := extractReportURLs(htmlContent, report)
		allReportURLs = append(allReportURLs, reportURLs...)

		// todo 测试环境跑部分数据，上线放开
		break
		// Check if next page button is disabled
		// todo 测试环境跑部分数据，上线放开
		/*var nextPageDisabled bool
		err = chromedp.Run(ctx,
			chromedp.Evaluate(`document.querySelector('div.my-page-next').classList.contains('my-page-forbid')`, &nextPageDisabled),
		)
		if err != nil {
			return err
		}

		if nextPageDisabled {
			break
		}

		// Click the next page button
		err = chromedp.Run(ctx,
			chromedp.Click(`div.my-page-next`, chromedp.ByQuery),
			chromedp.Sleep(5*time.Second),
			chromedp.Location(&categoryPageURL),
		)
		if err != nil {
			return err
		}
		*/
	}

	logs.Info("所有报告 URLs: %s: %s: %v", product, category, allReportURLs)

	if len(allReportURLs) == 0 {
		return fmt.Errorf("未找到报告 URL")
	}

	// 处理报告数据
	for _, reportURL := range allReportURLs {
		err = processReport(ctx, product, category, reportURL, keywords)
		if err != nil {
			logs.Error("处理报告错误: %s: %s: %s: %s: %v", product, category, report, reportURL, err)
		}
	}

	return nil
}

func fillProductPageURL(ctx context.Context, product string, category string) (string, error) {
	selector := `//dl[contains(@class, 'dl_hot')]`
	logs.Info("选择器表达式: %s", selector)

	var nodes []*cdp.Node
	var productPageURL string

	// 获取 dl 标签下的所有 a 标签节点
	err := chromedp.Run(ctx,
		chromedp.WaitReady(selector, chromedp.BySearch),
		chromedp.Nodes(selector+"//a", &nodes, chromedp.BySearch),
	)
	if err != nil {
		return "", err
	}

	// 提取并打印所有 a 标签的文本内容
	for _, node := range nodes {
		var linkText string
		err = chromedp.Run(ctx,
			chromedp.Text(node.FullXPath(), &linkText),
		)
		if err != nil {
			return "", err
		}
		logs.Info("Link Text: %s", linkText)
	}

	// 点击目标产品的链接
	clickSelector := fmt.Sprintf(`//dl[contains(@class, 'dl_hot')]//a[text()='%s']`, product)
	err = chromedp.Run(ctx,
		chromedp.WaitReady(clickSelector, chromedp.BySearch),
		chromedp.Click(clickSelector, chromedp.BySearch),
		chromedp.Sleep(5*time.Second),
		chromedp.Location(&productPageURL),
	)
	if err != nil {
		return "", err
	}

	// 返回点击后的页面URL
	logs.Info("productPageURL: %s", productPageURL)
	return productPageURL, nil
}

// Extract report URLs from the HTML content
func extractReportURLs(htmlContent, keyword string) []string {
	var reportURLs []string

	// Find all occurrences of the keyword and extract report URLs
	content := htmlContent
	for {
		startIdx := strings.Index(content, keyword)
		if startIdx == -1 {
			break
		}
		startIdx += len(keyword)

		// Extract the URL from the HTML content
		urlStartIdx := strings.LastIndex(content[:startIdx], `href="`) + len(`href="`)
		urlEndIdx := strings.Index(content[urlStartIdx:], `"`) + urlStartIdx
		if urlStartIdx > 0 && urlEndIdx > urlStartIdx {
			reportURLs = append(reportURLs, content[urlStartIdx:urlEndIdx])
		}

		content = content[startIdx:]
	}

	return reportURLs
}

// Process the report data
func processReport(ctx context.Context, product string, category string, reportURL string, keywords []string) error {
	// Navigate to the report page
	var reportContent string
	err := chromedp.Run(ctx,
		chromedp.Navigate(lyLoginPath+reportURL),
		chromedp.Sleep(5*time.Second),
		chromedp.OuterHTML("html", &reportContent),
	)
	if err != nil {
		return err
	}

	var lyIndexDataList []models.BaseFromLyData
	// Process the data based on keywords
	for _, keyword := range keywords {
		partialKeyword := strings.Split(keyword, "：")
		// Select appropriate processor based on product and category
		processor, err := GetProcessor(product, category)
		if err != nil {
			return err
		}

		// 查询报告所属分类
		classify, err := models.GetLyClassifyByName(product)
		if err != nil {
			return err
		}

		// Process the report content using the selected processor
		baseFromLyDataList, err := processor.Process(ctx, product, reportContent, partialKeyword, classify.BaseFromLyClassifyId)
		if err != nil {
			return err
		}
		if len(baseFromLyDataList) > 0 {
			for _, baseFromLyData := range baseFromLyDataList {
				if baseFromLyData.DataTime != "" && baseFromLyData.IndexCode != "" {
					baseFromLyData.CreateTime = utils.GetCurrentTime()
					baseFromLyData.ModifyTime = utils.GetCurrentTime()
					lyIndexDataList = append(lyIndexDataList, baseFromLyData)
				}
			}
		}

	}
	// 新增指标数据
	if len(lyIndexDataList) > 0 {
		err = models.AddLyDataList(lyIndexDataList)
		if err != nil {
			return err
		}
	}

	return nil
}

func addLyIndex(classifyId int, indexCode string, indexName string, unit string, frequency string) (int, error) {
	// 添加指标
	index := &models.BaseFromLyIndex{
		CreateTime:           utils.GetCurrentTime(),
		ModifyTime:           utils.GetCurrentTime(),
		BaseFromLyClassifyId: classifyId,
		IndexCode:            indexCode,
		IndexName:            indexName,
		Frequency:            frequency,
		Unit:                 unit,
		EdbExist:             0,
	}
	indexId, err := models.AddLyIndex(index)
	if err != nil {
		return 0, err
	}
	return int(indexId), nil
}
